@@ -13,6 +13,8 @@
  *-------
  */
 
+#include <stdlib.h>
+#include <string.h>
 #include "bind.h"
 
 #include "environ.h"
@@ -20,8 +22,7 @@
 #include "descriptor.h"
 #include "qresult.h"
 #include "pgtypes.h"
-#include <stdlib.h>
-#include <string.h>
+#include "multibyte.h"
 
 #include "pgapifunc.h"
 
@@ -34,11 +35,11 @@ PGAPI_BindParameter(
 					SQLSMALLINT fParamType,
 					SQLSMALLINT fCType,
 					SQLSMALLINT fSqlType,
-					SQLUINTEGER cbColDef,
+					SQLULEN cbColDef,
 					SQLSMALLINT ibScale,
 					PTR rgbValue,
-					SQLINTEGER cbValueMax,
-					SQLINTEGER *pcbValue)
+					SQLLEN cbValueMax,
+					SQLLEN FAR * pcbValue)
 {
 	StatementClass *stmt = (StatementClass *) hstmt;
 	CSTR func = "PGAPI_BindParameter";
@@ -71,7 +72,7 @@ PGAPI_BindParameter(
 	/* store the given info */
 	apdopts->parameters[ipar].buflen = cbValueMax;
 	apdopts->parameters[ipar].buffer = rgbValue;
-	apdopts->parameters[ipar].used = (Int4 *) pcbValue;
+	apdopts->parameters[ipar].used = pcbValue;
 	apdopts->parameters[ipar].CType = fCType;
 	ipdopts->parameters[ipar].SQLType = fSqlType;
 	ipdopts->parameters[ipar].paramType = fParamType;
@@ -79,8 +80,9 @@ PGAPI_BindParameter(
 	ipdopts->parameters[ipar].decimal_digits = ibScale;
 	ipdopts->parameters[ipar].precision = 0;
 	ipdopts->parameters[ipar].scale = 0;
-	ipdopts->parameters[ipar].PGType = sqltype_to_pgtype(stmt, fSqlType);
-
+	if (0 == ipdopts->parameters[ipar].PGType)
+		ipdopts->parameters[ipar].PGType = sqltype_to_pgtype(stmt, fSqlType);
+#if (ODBCVER >= 0x0300)
 	switch (fCType)
 	{
 		case SQL_C_NUMERIC:
@@ -96,6 +98,7 @@ PGAPI_BindParameter(
 	}
 	apdopts->parameters[ipar].precision = ipdopts->parameters[ipar].precision;
 	apdopts->parameters[ipar].scale = ipdopts->parameters[ipar].scale;
+#endif /* ODBCVER */
 
 	/*
 	 * If rebinding a parameter that had data-at-exec stuff in it, then
@@ -114,19 +117,24 @@ PGAPI_BindParameter(
 	}
 
 	if (pcbValue && apdopts->param_offset_ptr)
-		pcbValue += (*apdopts->param_offset_ptr >> 2);
+		pcbValue = LENADDR_SHIFT(pcbValue, *apdopts->param_offset_ptr);
+#ifdef	NOT_USED /* evaluation of pcbValue here is dangerous */
 	/* Data at exec macro only valid for C char/binary data */
 	if (pcbValue && (*pcbValue == SQL_DATA_AT_EXEC ||
 					 *pcbValue <= SQL_LEN_DATA_AT_EXEC_OFFSET))
 		apdopts->parameters[ipar].data_at_exec = TRUE;
 	else
 		apdopts->parameters[ipar].data_at_exec = FALSE;
+#endif /* NOT_USED */
 
 	/* Clear premature result */
 	if (stmt->status == STMT_PREMATURE)
 		SC_recycle_statement(stmt);
 
-	mylog("PGAPI_BindParamater: ipar=%d, paramType=%d, fCType=%d, fSqlType=%d, cbColDef=%d, ibScale=%d, rgbValue=%d, *pcbValue = %d, data_at_exec = %d\n", ipar, fParamType, fCType, fSqlType, cbColDef, ibScale, rgbValue, pcbValue ? *pcbValue : -777, apdopts->parameters[ipar].data_at_exec);
+	mylog("%s: ipar=%d, paramType=%d, fCType=%d, fSqlType=%d, cbColDef=%d, ibScale=%d,", func, ipar, fParamType, fCType, fSqlType, cbColDef, ibScale);
+	/*** mylog("rgbValue=%x, pcbValue = %x(%d), data_at_exec = %d\n", rgbValue, pcbValue, pcbValue ? *pcbValue : -777, apdopts->parameters[ipar].data_at_exec); ***/
+
+	mylog("rgbValue=%x, pcbValue = %x, data_at_exec = %d\n", rgbValue, pcbValue, apdopts->parameters[ipar].data_at_exec);
 
 	return SQL_SUCCESS;
 }
@@ -135,22 +143,23 @@ PGAPI_BindParameter(
 /*	Associate a user-supplied buffer with a database column. */
 RETCODE		SQL_API
 PGAPI_BindCol(
-			HSTMT hstmt,
-			SQLUSMALLINT icol, 
-			SQLSMALLINT fCType,
-			PTR rgbValue, 
-			SQLINTEGER cbValueMax,
-			SQLINTEGER *pcbValue)
+			  HSTMT hstmt,
+			  SQLUSMALLINT icol,
+			  SQLSMALLINT fCType,
+			  PTR rgbValue,
+			  SQLLEN cbValueMax,
+			  SQLLEN FAR * pcbValue)
 {
 	StatementClass *stmt = (StatementClass *) hstmt;
 	CSTR func = "PGAPI_BindCol";
 	ARDFields	*opts;
 	GetDataInfo	*gdata_info;
 	BindInfoClass	*bookmark;
+	RETCODE		ret = SQL_SUCCESS;
 
 	mylog("%s: entering...\n", func);
 
-	mylog("**** PGAPI_BindCol: stmt = %u, icol = %d\n", stmt, icol);
+	mylog("**** PGAPI_BindCol: stmt = %x, icol = %d\n", stmt, icol);
 	mylog("**** : fCType=%d rgb=%x valusMax=%d pcb=%x\n", fCType, rgbValue, cbValueMax, pcbValue);
 
 	if (!stmt)
@@ -162,11 +171,11 @@ PGAPI_BindCol(
 	opts = SC_get_ARDF(stmt);
 	if (stmt->status == STMT_EXECUTING)
 	{
-		SC_set_error(stmt, STMT_SEQUENCE_ERROR, "Can't bind columns while statement is still executing.");
-		SC_log_error(func, "", stmt);
+		SC_set_error(stmt, STMT_SEQUENCE_ERROR, "Can't bind columns while statement is still executing.", func);
 		return SQL_ERROR;
 	}
 
+#define	return	DONT_CALL_RETURN_FROM_HERE ???
 	SC_clear_error(stmt);
 	/* If the bookmark column is being bound, then just save it */
 	if (icol == 0)
@@ -186,23 +195,24 @@ PGAPI_BindCol(
 			switch (fCType)
 			{
 				case SQL_C_BOOKMARK:
+#if (ODBCVER >= 0x0300)
 				case SQL_C_VARBOOKMARK:
-
+#endif /* ODBCVER */
 					break;
 				default:
-					SC_set_error(stmt, STMT_PROGRAM_TYPE_OUT_OF_RANGE, "Bind column 0 is not of type SQL_C_BOOKMARK");
-                    mylog("Bind column 0 is type %d not of type SQL_C_BOOKMARK\n", fCType);
-					SC_log_error(func, "", stmt);
-					return SQL_ERROR;
+					SC_set_error(stmt, STMT_PROGRAM_TYPE_OUT_OF_RANGE, "Bind column 0 is not of type SQL_C_BOOKMARK", func);
+inolog("Bind column 0 is type %d not of type SQL_C_BOOKMARK", fCType);
+					ret = SQL_ERROR;
+					goto cleanup;
 			}
 
 			bookmark = ARD_AllocBookmark(opts);
 			bookmark->buffer = rgbValue;
-			bookmark->used = (Int4 *) pcbValue;
+			bookmark->used = pcbValue;
 			bookmark->buflen = cbValueMax;
 			bookmark->returntype = fCType;
 		}
-		return SQL_SUCCESS;
+		goto cleanup;
 	}
 
 	/*
@@ -219,9 +229,9 @@ PGAPI_BindCol(
 	/* check to see if the bindings were allocated */
 	if (!opts->bindings)
 	{
-		SC_set_error(stmt, STMT_NO_MEMORY_ERROR, "Could not allocate memory for bindings.");
-		SC_log_error(func, "", stmt);
-		return SQL_ERROR;
+		SC_set_error(stmt, STMT_NO_MEMORY_ERROR, "Could not allocate memory for bindings.", func);
+		ret = SQL_ERROR;
+		goto cleanup;
 	}
 
 	/* use zero based col numbers from here out */
@@ -250,19 +260,24 @@ PGAPI_BindCol(
 		/* ok, bind that column */
 		opts->bindings[icol].buflen = cbValueMax;
 		opts->bindings[icol].buffer = rgbValue;
-		opts->bindings[icol].used = (Int4 *) pcbValue;
+		opts->bindings[icol].used = pcbValue;
 		opts->bindings[icol].returntype = fCType;
-
+#if (ODBCVER >= 0x0300)
 		if (SQL_C_NUMERIC == fCType)
 			opts->bindings[icol].precision = 32;
 		else
+#endif /* ODBCVER */
 			opts->bindings[icol].precision = 0;
 		opts->bindings[icol].scale = 0;
 
-		mylog("       bound buffer[%d] = %u\n", icol, opts->bindings[icol].buffer);
+		mylog("       bound buffer[%d] = %x\n", icol, opts->bindings[icol].buffer);
 	}
 
-	return SQL_SUCCESS;
+cleanup:
+#undef	return
+	if (stmt->internal)
+		ret = DiscardStatementSvp(stmt, ret, FALSE);
+	return ret;
 }
 
 
@@ -275,19 +290,22 @@ PGAPI_BindCol(
  *	data type (most likely varchar).
  */
 RETCODE		SQL_API
-PGAPI_DescribeParam(HSTMT hstmt,
-			SQLUSMALLINT ipar,
-			SQLSMALLINT *pfSqlType,
-			SQLUINTEGER *pcbColDef,
-			SQLSMALLINT *pibScale,
-			SQLSMALLINT *pfNullable)
+PGAPI_DescribeParam(
+					HSTMT hstmt,
+					SQLUSMALLINT ipar,
+					SQLSMALLINT FAR * pfSqlType,
+					SQLULEN FAR * pcbParamDef,
+					SQLSMALLINT FAR * pibScale,
+					SQLSMALLINT FAR * pfNullable)
 {
 	StatementClass *stmt = (StatementClass *) hstmt;
 	CSTR func = "PGAPI_DescribeParam";
 	APDFields	*apdopts;
 	IPDFields	*ipdopts;
+	RETCODE		ret = SQL_SUCCESS;
+	int		num_params;
 
-	mylog("%s: entering...\n", func);
+	mylog("%s: entering...%d\n", func, ipar);
 
 	if (!stmt)
 	{
@@ -297,42 +315,89 @@ PGAPI_DescribeParam(HSTMT hstmt,
 	SC_clear_error(stmt);
 
 	apdopts = SC_get_APDF(stmt);
-	if ((ipar < 1) || (ipar > apdopts->allocated))
+	ipdopts = SC_get_IPDF(stmt);
+	/*if ((ipar < 1) || (ipar > ipdopts->allocated))*/
+	num_params = stmt->num_params;
+	if (num_params < 0)
 	{
-		SC_set_error(stmt, STMT_BAD_PARAMETER_NUMBER_ERROR, "Invalid parameter number for PGAPI_DescribeParam.");
-		SC_log_error(func, "", stmt);
+		SQLSMALLINT	num_p;
+
+		PGAPI_NumParams(stmt, &num_p);
+		num_params = num_p;
+	}
+	if ((ipar < 1) || (ipar > num_params))
+	{
+inolog("num_params=%d\n", stmt->num_params);
+		SC_set_error(stmt, STMT_BAD_PARAMETER_NUMBER_ERROR, "Invalid parameter number for PGAPI_DescribeParam.", func);
 		return SQL_ERROR;
 	}
-	ipdopts = SC_get_IPDF(stmt);
+	extend_iparameter_bindings(ipdopts, stmt->num_params);
+
+#define	return	DONT_CALL_RETURN_FROM_HERE???
+	/* StartRollbackState(stmt); */
+	if (NOT_YET_PREPARED == stmt->prepared)
+	{
+		decideHowToPrepare(stmt);
+inolog("howTo=%d\n", SC_get_prepare_method(stmt));
+		switch (SC_get_prepare_method(stmt))
+		{
+			case USING_PARSE_REQUEST:
+			case USING_UNNAMED_PARSE_REQUEST:
+				if (ret = prepareParameters(stmt), SQL_ERROR == ret)
+					goto cleanup;
+		}
+	}
 
 	ipar--;
-
 	/*
 	 * This implementation is not very good, since it is supposed to
 	 * describe
 	 */
 	/* parameter markers, not bound parameters.  */
 	if (pfSqlType)
-		*pfSqlType = ipdopts->parameters[ipar].SQLType;
+	{
+inolog("[%d].SQLType=%d .PGType=%d\n", ipar, ipdopts->parameters[ipar].SQLType,
+ipdopts->parameters[ipar].PGType);
+		if (ipdopts->parameters[ipar].SQLType)
+			*pfSqlType = ipdopts->parameters[ipar].SQLType;
+		else if (ipdopts->parameters[ipar].PGType)
+			*pfSqlType = pgtype_to_concise_type(stmt, ipdopts->parameters[ipar].PGType, PG_STATIC);
+	}
 
-	if (pcbColDef)
-		*pcbColDef = ipdopts->parameters[ipar].column_size;
+	if (pcbParamDef)
+	{
+		*pcbParamDef = 0;
+		if (ipdopts->parameters[ipar].SQLType)
+			*pcbParamDef = ipdopts->parameters[ipar].column_size;
+		if (0 == *pcbParamDef && ipdopts->parameters[ipar].PGType)
+			*pcbParamDef = pgtype_column_size(stmt, ipdopts->parameters[ipar].PGType, PG_STATIC, PG_STATIC);
+	}
 
 	if (pibScale)
-		*pibScale = ipdopts->parameters[ipar].decimal_digits;
+	{
+		*pibScale = 0;
+		if (ipdopts->parameters[ipar].SQLType)
+			*pibScale = ipdopts->parameters[ipar].decimal_digits;
+		else if (ipdopts->parameters[ipar].PGType)
+			*pibScale = pgtype_scale(stmt, ipdopts->parameters[ipar].PGType, -1);
+	}
 
 	if (pfNullable)
 		*pfNullable = pgtype_nullable(stmt, ipdopts->parameters[ipar].paramType);
-
-	return SQL_SUCCESS;
+cleanup:
+#undef	return
+	if (stmt->internal)
+		ret = DiscardStatementSvp(stmt, ret, FALSE);
+	return ret;
 }
 
 
 /*	Sets multiple values (arrays) for the set of parameter markers. */
 RETCODE		SQL_API
-PGAPI_ParamOptions(HSTMT hstmt,
-		   SQLUINTEGER crow,
-		   SQLUINTEGER *pirow)
+PGAPI_ParamOptions(
+				   HSTMT hstmt,
+				   SQLULEN crow,
+				   SQLULEN FAR * pirow)
 {
 	CSTR func = "PGAPI_ParamOptions";
 	StatementClass *stmt = (StatementClass *) hstmt;
@@ -359,12 +424,11 @@ PGAPI_ParamOptions(HSTMT hstmt,
 RETCODE		SQL_API
 PGAPI_NumParams(
 				HSTMT hstmt,
-				SWORD FAR * pcpar)
+				SQLSMALLINT FAR * pcpar)
 {
 	StatementClass *stmt = (StatementClass *) hstmt;
-	char		in_quote = FALSE;
-	unsigned int i;
 	CSTR func = "PGAPI_NumParams";
+	char	literal_quote = LITERAL_QUOTE, identifier_quote = IDENTIFIER_QUOTE, dollar_quote = DOLLAR_QUOTE;
 
 	mylog("%s: entering...\n", func);
 
@@ -378,34 +442,117 @@ PGAPI_NumParams(
 		*pcpar = 0;
 	else
 	{
-		SC_log_error(func, "pcpar was null", stmt);
+		SC_set_error(stmt, STMT_EXEC_ERROR, "parameter count address is null", func);
 		return SQL_ERROR;
 	}
-
-
+inolog("num_params=%d,%d\n", stmt->num_params, stmt->proc_return);
 	if (stmt->num_params >= 0)
 		*pcpar = stmt->num_params;
 	else if (!stmt->statement)
 	{
 		/* no statement has been allocated */
-		SC_set_error(stmt, STMT_SEQUENCE_ERROR, "PGAPI_NumParams called with no statement ready.");
-		SC_log_error(func, "", stmt);
+		SC_set_error(stmt, STMT_SEQUENCE_ERROR, "PGAPI_NumParams called with no statement ready.", func);
 		return SQL_ERROR;
 	}
 	else
 	{
-		for (i = 0; i < strlen(stmt->statement); i++)
+		const	char *sptr, *tag = NULL;
+		ConnectionClass	*conn = SC_get_conn(stmt);
+		int	taglen;
+		char	tchar, bchar, escape_in_literal;
+		char	in_literal = FALSE, in_identifier = FALSE,
+			in_dollar_quote = FALSE, in_escape = FALSE,
+			multi = FALSE, del_found = FALSE;
+		encoded_str	encstr;
+
+		stmt->proc_return = 0;
+		make_encoded_str(&encstr, conn, stmt->statement);
+		for (sptr = stmt->statement, bchar = '\0'; *sptr; sptr++)
 		{
-			if (stmt->statement[i] == '?' && !in_quote)
-				(*pcpar)++;
+			tchar = encoded_nextchar(&encstr);
+			if (ENCODE_STATUS(encstr) != 0) /* multibyte char */
+			{
+				if ((UCHAR) tchar >= 0x80)
+					bchar = tchar;
+				continue;
+			}
+			if (!multi && del_found)
+			{
+				if (!isspace(tchar))
+					multi = TRUE;
+			}
+			if (in_dollar_quote)
+			{
+				if (tchar == dollar_quote)
+				{
+					if (strncmp(sptr, tag, taglen) == 0)
+					{
+						in_dollar_quote = FALSE;
+						tag = NULL;
+						sptr += taglen;
+						sptr--;
+						encoded_position_shift(&encstr, taglen - 1);
+					}
+				}
+			}
+			else if (in_literal)
+			{
+				if (in_escape)
+					in_escape = FALSE;
+				else if (tchar == escape_in_literal)
+					in_escape = TRUE;
+				else if (tchar == literal_quote)
+					in_literal = FALSE;
+			}
+			else if (in_identifier)
+			{
+				if (tchar == identifier_quote)
+					in_identifier = FALSE;
+			}
 			else
 			{
-				if (stmt->statement[i] == '\'')
-					in_quote = (in_quote ? FALSE : TRUE);
+				if (tchar == '?')
+				{
+					if (0 == *pcpar && bchar == '{')
+						stmt->proc_return = 1;
+					(*pcpar)++;
+				}
+				else if (tchar == ';')
+					del_found = TRUE;
+				else if (tchar == dollar_quote)
+				{
+					char	*dollar_next;
+
+					in_dollar_quote = TRUE;
+					tag = sptr;
+					taglen = 0; 
+					if (dollar_next = strchr(sptr + 1, dollar_quote))
+					{
+						taglen = dollar_next - sptr + 1;
+						sptr = dollar_next;
+						encoded_position_shift(&encstr, taglen - 1);
+					}
+				}
+				else if (tchar == literal_quote)
+				{
+					in_literal = TRUE;
+					escape_in_literal = CC_get_escape(conn);
+					if (!escape_in_literal)
+					{
+						if (LITERAL_EXT == sptr[-1])
+							escape_in_literal = ESCAPE_IN_LITERAL;
+					}
+				}
+				else if (tchar == identifier_quote)
+					in_identifier = TRUE;
+				if (!isspace(tchar))
+					bchar = tchar;
 			}
 		}
 		stmt->num_params = *pcpar;
+		stmt->multi_statement = multi;
 	}
+inolog("num_params=%d,%d\n", stmt->num_params, stmt->proc_return);
 	return SQL_SUCCESS;
 }
 
@@ -439,7 +586,7 @@ extend_parameter_bindings(APDFields *self, int num_params)
 	CSTR func = "extend_parameter_bindings";
 	ParameterInfoClass *new_bindings;
 
-	mylog("%s: entering ... self=%u, parameters_allocated=%d, num_params=%d\n", func, self, self->allocated, num_params);
+	mylog("%s: entering ... self=%x, parameters_allocated=%d, num_params=%d\n", func, self, self->allocated, num_params);
 
 	/*
 	 * if we have too few, allocate room for more, and copy the old
@@ -472,7 +619,7 @@ extend_iparameter_bindings(IPDFields *self, int num_params)
 	CSTR func = "extend_iparameter_bindings";
 	ParameterImplClass *new_bindings;
 
-	mylog("%s: entering ... self=%u, parameters_allocated=%d, num_params=%d\n", func, self, self->allocated, num_params);
+	mylog("%s: entering ... self=%x, parameters_allocated=%d, num_params=%d\n", func, self, self->allocated, num_params);
 
 	/*
 	 * if we have too few, allocate room for more, and copy the old
@@ -504,7 +651,7 @@ reset_a_parameter_binding(APDFields *self, int ipar)
 {
 	CSTR func = "reset_a_parameter_binding";
 
-	mylog("%s: entering ... self=%u, parameters_allocated=%d, ipar=%d\n", func, self, self->allocated, ipar);
+	mylog("%s: entering ... self=%x, parameters_allocated=%d, ipar=%d\n", func, self, self->allocated, ipar);
 
 	if (ipar < 1 || ipar > self->allocated)
 		return;
@@ -524,18 +671,64 @@ reset_a_iparameter_binding(IPDFields *self, int ipar)
 {
 	CSTR func = "reset_a_iparameter_binding";
 
-	mylog("%s: entering ... self=%u, parameters_allocated=%d, ipar=%d\n", func, self, self->allocated, ipar);
+	mylog("%s: entering ... self=%x, parameters_allocated=%d, ipar=%d\n", func, self, self->allocated, ipar);
 
 	if (ipar < 1 || ipar > self->allocated)
 		return;
 
 	ipar--;
+	NULL_THE_NAME(self->parameters[ipar].paramName);
 	self->parameters[ipar].paramType = 0;
 	self->parameters[ipar].SQLType = 0;
 	self->parameters[ipar].column_size = 0;
 	self->parameters[ipar].decimal_digits = 0;
 	self->parameters[ipar].precision = 0;
 	self->parameters[ipar].scale = 0;
+	self->parameters[ipar].PGType = 0;
+}
+
+int
+CountParameters(const StatementClass *self, Int2 *inputCount, Int2 *ioCount, Int2 *outputCount)
+{
+	CSTR func = "CountParameters";
+	IPDFields	*ipdopts = SC_get_IPDF(self);
+	int	i, num_params, valid_count;
+
+	if (inputCount)
+		*inputCount = 0;
+	if (ioCount)
+		*ioCount = 0;
+	if (outputCount)
+		*outputCount = 0;
+	if (!ipdopts)	return -1;
+	num_params = self->num_params;
+	if (ipdopts->allocated < num_params)
+		num_params = ipdopts->allocated;
+	for (i = 0, valid_count = 0; i < num_params; i++)
+	{
+		if (SQL_PARAM_OUTPUT == ipdopts->parameters[i].paramType)
+		{
+			if (outputCount)
+			{
+				(*outputCount)++;
+				valid_count++;
+			}
+		}
+		else if (SQL_PARAM_INPUT_OUTPUT == ipdopts->parameters[i].paramType)
+		{
+			if (ioCount)
+			{
+				(*ioCount)++;
+				valid_count++;
+			}
+		}
+		else if (inputCount)
+		{
+			 (*inputCount)++;
+			valid_count++;
+		}
+	}
+	return valid_count;
 }
 
 /*
@@ -564,7 +757,7 @@ PDATA_free_params(PutDataInfo *pdata, char option)
 {
 	int			i;
 
-	mylog("PDATA_free_params:  ENTER, self=%d\n", pdata);
+	mylog("PDATA_free_params:  ENTER, self=%x\n", pdata);
 
 	if (!pdata->pdata)
 		return;
@@ -599,7 +792,7 @@ PDATA_free_params(PutDataInfo *pdata, char option)
 void
 IPD_free_params(IPDFields *ipdopts, char option)
 {
-	mylog("IPD_free_params:  ENTER, self=%d\n", ipdopts);
+	mylog("IPD_free_params:  ENTER, self=%x\n", ipdopts);
 
 	if (!ipdopts->parameters)
 		return;
@@ -620,7 +813,7 @@ extend_column_bindings(ARDFields *self, int num_columns)
 	BindInfoClass *new_bindings;
 	int			i;
 
-	mylog("%s: entering ... self=%u, bindings_allocated=%d, num_columns=%d\n", func, self, self->allocated, num_columns);
+	mylog("%s: entering ... self=%x, bindings_allocated=%d, num_columns=%d\n", func, self, self->allocated, num_columns);
 
 	/*
 	 * if we have too few, allocate room for more, and copy the old
@@ -673,7 +866,7 @@ reset_a_column_binding(ARDFields *self, int icol)
 	CSTR func = "reset_a_column_binding";
 	BindInfoClass	*bookmark;
 
-	mylog("%s: entering ... self=%u, bindings_allocated=%d, icol=%d\n", func, self, self->allocated, icol);
+	mylog("%s: entering ... self=%x, bindings_allocated=%d, icol=%d\n", func, self, self->allocated, icol);
 
 	if (icol > self->allocated)
 		return;
@@ -703,7 +896,7 @@ void	ARD_unbind_cols(ARDFields *self, BOOL freeall)
 {
 	Int2	lf;
 
-    mylog("ARD_unbind_cols freeall=%d allocated=%d bindings=%x\n", freeall, self->allocated, self->bindings);
+inolog("ARD_unbind_cols freeall=%d allocated=%d bindings=%x", freeall, self->allocated, self->bindings);
 	for (lf = 1; lf <= self->allocated; lf++)
 		reset_a_column_binding(self, lf);
 	if (freeall)
@@ -718,7 +911,7 @@ void	GDATA_unbind_cols(GetDataInfo *self, BOOL freeall)
 {
 	Int2	lf;
 
-    mylog("GDATA_unbind_cols freeall=%d allocated=%d gdata=%x\n", freeall, self->allocated, self->gdata);
+inolog("GDATA_unbind_cols freeall=%d allocated=%d gdata=%x", freeall, self->allocated, self->gdata);
 	if (self->fdata.ttlbuf)
 	{
 		free(self->fdata.ttlbuf);
@@ -772,7 +965,7 @@ extend_getdata_info(GetDataInfo *self, int num_columns, BOOL shrink)
 	GetDataClass	*new_gdata;
 	int			i;
 
-	mylog("%s: entering ... self=%u, gdata_allocated=%d, num_columns=%d\n", func, self, self->allocated, num_columns);
+	mylog("%s: entering ... self=%x, gdata_allocated=%d, num_columns=%d\n", func, self, self->allocated, num_columns);
 
 	/*
 	 * if we have too few, allocate room for more, and copy the old
@@ -848,7 +1041,7 @@ extend_putdata_info(PutDataInfo *self, int num_params, BOOL shrink)
 	CSTR func = "extend_putdata_info";
 	PutDataClass	*new_pdata;
 
-	mylog("%s: entering ... self=%u, parameters_allocated=%d, num_params=%d\n", func, self, self->allocated, num_params);
+	mylog("%s: entering ... self=%x, parameters_allocated=%d, num_params=%d\n", func, self, self->allocated, num_params);
 
 	/*
 	 * if we have too few, allocate room for more, and copy the old
@@ -856,6 +1049,11 @@ extend_putdata_info(PutDataInfo *self, int num_params, BOOL shrink)
 	 */
 	if (self->allocated < num_params)
 	{
+		if (self->allocated <= 0 && self->pdata)
+		{
+			mylog("??? pdata is not null while allocated == 0\n");
+			self->pdata = NULL; 
+		}
 		new_pdata = (PutDataClass *) realloc(self->pdata, sizeof(PutDataClass) * num_params);
 		if (!new_pdata)
 		{
@@ -904,3 +1102,34 @@ void	reset_a_putdata_info(PutDataInfo *pdata_info, int ipar)
 	}
 	pdata_info->pdata[ipar].lobj_oid = 0;
 }
+
+void SC_param_next(const StatementClass *stmt, int *param_number, ParameterInfoClass **apara, ParameterImplClass **ipara)
+{
+	int	next;
+	IPDFields	*ipdopts = SC_get_IPDF(stmt);
+
+	if (*param_number < 0)
+		next = stmt->proc_return;
+	else
+		next = *param_number + 1;
+	if (stmt->discard_output_params)
+	{
+		for (;next < ipdopts->allocated && SQL_PARAM_OUTPUT == ipdopts->parameters[next].paramType; next++) ;
+	}
+	*param_number = next;
+	if (ipara)
+	{
+		if (next < ipdopts->allocated)
+			*ipara = ipdopts->parameters + next;
+		else
+			*ipara = NULL;
+	}
+	if (apara)
+	{
+		APDFields	*apdopts = SC_get_APDF(stmt);
+		if (next < apdopts->allocated)
+			*apara = apdopts->parameters + next;
+		else
+			*apara = NULL;
+	}
+}
