@@ -17,201 +17,19 @@
 #include <stdio.h>
 #include <stdarg.h>
 #include <string.h>
-#include <ctype.h>
+#include <time.h>
 
 #ifndef WIN32
 #include <pwd.h>
 #include <sys/types.h>
-#include <sys/stat.h>
-#include <fcntl.h>
 #include <unistd.h>
 #else
 #include <process.h>			/* Byron: is this where Windows keeps def.
 								 * of getpid ? */
 #endif
-
 #include "connection.h"
 #include "multibyte.h"
 
-extern GLOBAL_VALUES globals;
-void		generate_filename(const char *, const char *, char *);
-
-
-void
-generate_filename(const char *dirname, const char *prefix, char *filename)
-{
-	int			pid = 0;
-
-#ifndef WIN32
-	struct passwd *ptr = 0;
-
-	ptr = getpwuid(getuid());
-#endif
-	pid = getpid();
-	if (dirname == 0 || filename == 0)
-		return;
-
-	strcpy(filename, dirname);
-	strcat(filename, DIRSEPARATOR);
-	if (prefix != 0)
-		strcat(filename, prefix);
-#ifndef WIN32
-	strcat(filename, ptr->pw_name);
-#endif
-	sprintf(filename, "%s%u%s", filename, pid, ".log");
-	return;
-}
-
-#if defined(WIN_MULTITHREAD_SUPPORT)
-CRITICAL_SECTION	qlog_cs, mylog_cs;
-#elif defined(POSIX_MULTITHREAD_SUPPORT)
-pthread_mutex_t	qlog_cs, mylog_cs;
-#endif /* WIN_MULTITHREAD_SUPPORT */
-static int	mylog_on = 0,
-			qlog_on = 0;
-
-int	get_mylog(void)
-{
-	return mylog_on;
-}
-int	get_qlog(void)
-{
-	return qlog_on;
-}
-
-void
-logs_on_off(int cnopen, int mylog_onoff, int qlog_onoff)
-{
-	static int	mylog_on_count = 0,
-				mylog_off_count = 0,
-				qlog_on_count = 0,
-				qlog_off_count = 0;
-
-	ENTER_MYLOG_CS;
-	ENTER_QLOG_CS;
-	if (mylog_onoff)
-		mylog_on_count += cnopen;
-	else
-		mylog_off_count += cnopen;
-	if (mylog_on_count > 0)
-		mylog_on = 1;
-	else if (mylog_off_count > 0)
-		mylog_on = 0;
-	else
-		mylog_on = globals.debug;
-	if (qlog_onoff)
-		qlog_on_count += cnopen;
-	else
-		qlog_off_count += cnopen;
-	if (qlog_on_count > 0)
-		qlog_on = 1;
-	else if (qlog_off_count > 0)
-		qlog_on = 0;
-	else
-		qlog_on = globals.commlog;
-	LEAVE_QLOG_CS;
-	LEAVE_MYLOG_CS;
-}
-
-#ifdef MY_LOG
-static FILE *LOGFP = NULL;
-void
-mylog(char *fmt,...)
-{
-	va_list		args;
-	char		filebuf[80];
-
-#ifndef WIN32
-	int		filedes=0;
-#endif
-
-	ENTER_MYLOG_CS;
-	if (mylog_on)
-	{
-		va_start(args, fmt);
-
-		if (!LOGFP)
-		{
-			generate_filename(MYLOGDIR, MYLOGFILE, filebuf);
-#ifdef WIN32
-			LOGFP = fopen(filebuf, PG_BINARY_A);
-#else
-			filedes = open(filebuf, O_WRONLY | O_APPEND | O_CREAT, S_IWUSR | S_IRUSR);
-			LOGFP = fdopen(filedes, PG_BINARY_A);
-#endif
-			if (LOGFP)
-				setbuf(LOGFP, NULL);
-			else
-				mylog_on = 0;
-		}
-
-#ifdef	WIN_MULTITHREAD_SUPPORT
-#ifdef	WIN32
-		if (LOGFP)
-			fprintf(LOGFP, "[%d]", GetCurrentThreadId());
-#endif /* WIN32 */
-#endif /* WIN_MULTITHREAD_SUPPORT */
-#if defined(POSIX_MULTITHREAD_SUPPORT)
-		if (LOGFP)
-			fprintf(LOGFP, "[%d]", pthread_self());
-#endif /* POSIX_MULTITHREAD_SUPPORT */
-		if (LOGFP)
-			vfprintf(LOGFP, fmt, args);
-
-		va_end(args);
-	}
-	LEAVE_MYLOG_CS;
-}
-#else
-void
-MyLog(char *fmt,...)
-{
-}
-#endif
-
-
-#ifdef Q_LOG
-void
-qlog(char *fmt,...)
-{
-	va_list		args;
-	char		filebuf[80];
-	static FILE *LOGFP = NULL;
-
-#ifndef WIN32
-        int             filedes=0;
-#endif
-
-	ENTER_QLOG_CS;
-	if (qlog_on)
-	{
-		va_start(args, fmt);
-
-		if (!LOGFP)
-		{
-			generate_filename(QLOGDIR, QLOGFILE, filebuf);
-#ifdef WIN32
-			LOGFP = fopen(filebuf, PG_BINARY_A);
-#else
-			filedes = open(filebuf, O_WRONLY | O_APPEND | O_CREAT, S_IWUSR | S_IRUSR);
-			LOGFP = fdopen(filedes, PG_BINARY_A);
-#endif
-			if (LOGFP)
-				setbuf(LOGFP, NULL);
-			else
-				qlog_on = 0;
-		}
-
-		if (LOGFP)
-			vfprintf(LOGFP, fmt, args);
-
-		va_end(args);
-	}
-	LEAVE_QLOG_CS;
-}
-#endif
-
-
 /*
  *	returns STRCPY_FAIL, STRCPY_TRUNCATED, or #bytes copied
  *	(not including null term)
@@ -287,21 +105,19 @@ strncpy_null(char *dst, const char *src, int len)
 /*------
  *	Create a null terminated string (handling the SQL_NTS thing):
  *		1. If buf is supplied, place the string in there
- *		   (at most bufsize-1 bytes) and return buf.
+ *		   (assumes enough space) and return buf.
  *		2. If buf is not supplied, malloc space and return this string
- *		   (buflen is ignored in this case)
  *------
  */
 char *
 make_string(const char *s, int len, char *buf, size_t bufsize)
 {
-	unsigned int	length;
+	int			length;
 	char	   *str;
 
 	if (s && (len > 0 || (len == SQL_NTS && strlen(s) > 0)))
 	{
 		length = (len > 0) ? len : strlen(s);
-
 		if (buf)
 		{
 			if (length >= bufsize)
@@ -310,7 +126,9 @@ make_string(const char *s, int len, char *buf, size_t bufsize)
 			return buf;
 		}
 
+inolog("malloc size=%d\n", length);
 		str = malloc(length + 1);
+inolog("str=%x\n", str);
 		if (!str)
 			return NULL;
 
@@ -353,7 +171,7 @@ make_lstring_ifneeded(ConnectionClass *conn, const char *s, int len, BOOL ifallu
 					str = NULL;
 				}
 				break;
-			}
+			} 
 			if (tolower(*ptr) != *ptr)
 			{
 				if (!str)
@@ -413,12 +231,12 @@ schema_strcat(char *buf, const char *fmt, const char *s, int len, const char *tb
 void
 remove_newlines(char *string)
 {
-	unsigned int i;
+	unsigned int i, len = strlen(string);
 
-	for (i = 0; i < strlen(string); i++)
+	for (i = 0; i < len; i++)
 	{
-		if ((string[i] == '\n') ||
-			(string[i] == '\r'))
+		if ((PG_LINEFEED == string[i]) ||
+			(PG_CARRIAGE_RETURN == string[i]))
 			string[i] = ' ';
 	}
 }
@@ -440,6 +258,10 @@ trim(char *s)
 	return s;
 }
 
+/*
+ *	my_strcat1 is a extension of my_strcat.
+ *	It can have 1 more parameter than my_strcat.
+ */
 char *
 my_strcat1(char *buf, const char *fmt, const char *s1, const char *s, int len)
 {
@@ -470,19 +292,38 @@ schema_strcat1(char *buf, const char *fmt, const char *s1, const char *s, int le
 	return my_strcat1(buf, fmt, s1, s, len);
 }
 
-int 
-contains_token(char *data, char *token)
+/*
+ * snprintf_add is a extension to snprintf
+ * It add format to buf at given pos
+ */
+
+int
+snprintf_add(char *buf, size_t size, const char *format, ...)
 {
-	int	i, tlen, dlen;
+	int len;
+	size_t pos = strlen(buf);
+	va_list arglist;
+	
+	va_start(arglist, format);
+	len = vsnprintf(buf + pos, size - pos, format, arglist);
+	va_end(arglist);
+	return len;
+}
 
-	dlen = strlen(data);
-	tlen = strlen(token);
+/*
+ * snprintf_addlen is a extension to snprintf
+ * It returns strlen of buf every time (not -1 when truncated)
+ */
 
-	for (i = 0; i < dlen-tlen+1; i++)
-	{
-		if (!strnicmp((const char *)data+i, token, tlen))
-			return 1;
-	}
-
-	return 0;
+int
+snprintf_len(char *buf, size_t size, const char *format, ...)
+{
+	int len;
+	va_list arglist;
+	
+	va_start(arglist, format);
+	if ((len = vsnprintf(buf, size, format, arglist)) < 0)
+		len = size;
+	va_end(arglist);
+	return len;
 }
