@@ -9,49 +9,48 @@
 #include "qresult.h"
 
 /* PostgreSQL client encoding */
-#define SQL_ASCII			0	/* SQL/ASCII */
-#define EUC_JP				1	/* EUC for Japanese */
-#define EUC_CN				2	/* EUC for Chinese */
-#define EUC_KR				3	/* EUC for Korean */
-#define EUC_TW				4	/* EUC for Taiwan */
-#define JOHAB				5
-#define UTF8				6	/* Unicode UTF-8 */
-#define MULE_INTERNAL		7	/* Mule internal code */
-#define LATIN1				8	/* ISO-8859 Latin 1 */
-#define LATIN2				9	/* ISO-8859 Latin 2 */
-#define LATIN3				10	/* ISO-8859 Latin 3 */
-#define LATIN4				11	/* ISO-8859 Latin 4 */
-#define LATIN5				12	/* ISO-8859 Latin 5 */
-#define LATIN6				13	/* ISO-8859 Latin 6 */
-#define LATIN7				14	/* ISO-8859 Latin 7 */
-#define LATIN8				15	/* ISO-8859 Latin 8 */
-#define LATIN9				16	/* ISO-8859 Latin 9 */
-#define LATIN10				17	/* ISO-8859 Latin 10 */
-#define WIN1256				18	/* Arabic Windows */
-#define WIN1258				19	/* Vietnamese Windows */
-#define WIN874				20	/* Thai Windows */
-#define KOI8R				21	/* KOI8-R/U */
-#define WIN1251				22	/* windows-1251 */
-#define WIN866				23	/* Alternativny Variant (MS-DOS CP866) */
-#define ISO_8859_5			24	/* ISO-8859-5 */
-#define ISO_8859_6			25	/* ISO-8859-6 */
-#define ISO_8859_7			26	/* ISO-8859-7 */
-#define ISO_8859_8			27	/* ISO-8859-8 */
-
-#define SJIS				28	/* Shift JIS */
-#define BIG5				29	/* Big5 */
-#define GBK					30	/* GBK */
-#define UHC					31  /* UHC */
-#define WIN1250				32	/* windows-1250 */
-#define GB18030				33	/* GB18030 */
-
-#define PG_UNICODE			34  /* UNICODE ( < Ver8.1) */
-					    /* Can't call it UNICODE as that's already used */
-#define TCVN				35  /* TCVN ( < Ver8.1) */
-#define ALT				36  /* ALT ( < Var8.1) */
-#define WIN				37  /* WIN ( < Ver8.1) */
-
-#define OTHER				-1
+enum {
+	SQL_ASCII 	= 0	/* SQL/ASCII */
+	,EUC_JP			/* EUC for Japanese */
+	,EUC_CN			/* EUC for Chinese */
+	,EUC_KR			/* EUC for Korean */
+	,EUC_TW			/* EUC for Taiwan */
+	,JOHAB
+	,UTF8			/* Unicode UTF-8 */
+	,MULE_INTERNAL		/* Mule internal code */
+	,LATIN1			/* ISO-8859 Latin 1 */
+	,LATIN2			/* ISO-8859 Latin 2 */
+	,LATIN3			/* ISO-8859 Latin 3 */
+	,LATIN4			/* ISO-8859 Latin 4 */
+	,LATIN5			/* ISO-8859 Latin 5 */
+	,LATIN6			/* ISO-8859 Latin 6 */
+	,LATIN7			/* ISO-8859 Latin 7 */
+	,LATIN8			/* ISO-8859 Latin 8 */
+	,LATIN9			/* ISO-8859 Latin 9 */
+	,LATIN10		/* ISO-8859 Latin 10 */
+	,WIN1256		/* Arabic Windows */
+	,WIN1258		/* Vietnamese Windows */
+	,WIN874			/* Thai Windows */
+	,KOI8R			/* KOI8-R/U */
+	,WIN1251		/* windows-1251 */
+	,WIN866			/* Alternativny Variant (MS-DOS CP866) */
+	,ISO_8859_5		/* ISO-8859-5 */
+	,ISO_8859_6		/* ISO-8859-6 */
+	,ISO_8859_7		/* ISO-8859-7 */
+	,ISO_8859_8		/* ISO-8859-8 */
+	,SJIS			/* Shift JIS */
+	,BIG5			/* Big5 */
+	,GBK			/* GBK */
+	,UHC			/* UHC */
+	,WIN1250		/* windows-1250 */
+	,GB18030		/* GB18030 */
+	,UNICODE_PODBC		/* same as Unicode UTF-8 */
+	,TCVN			/* TCVN ( < Ver8.1) */
+	,ALT			/* ALT ( < Var8.1) */
+	,WIN			/* WIN ( < Ver8.1) */
+	,WIN1252		/* WIN1252 */
+	,OTHER		=	-1
+};
 
 #define MAX_CHARACTERSET_NAME	24
 #define MAX_CHARACTER_LEN	6
@@ -68,6 +67,7 @@
 /* New Type */
 
 extern void CC_lookup_characterset(ConnectionClass *self);
+extern const char *get_environment_encoding(const ConnectionClass *conn, const char *oldenc, const char *svrenc);
 
 extern int pg_CS_stat(int stat,unsigned int charcter,int characterset_code);
 extern int pg_CS_code(const UCHAR *stat_string);
@@ -95,6 +95,7 @@ typedef struct
 void encoded_str_constr(encoded_str *encstr, int ccsc, const char *str);
 #define make_encoded_str(encstr, conn, str) encoded_str_constr(encstr, conn->ccsc, str)
 extern int encoded_nextchar(encoded_str *encstr);
+extern int encoded_position_shift(encoded_str *encstr, int shift);
 extern int encoded_byte_check(encoded_str *encstr, int abspos);
 #define check_client_encoding(X) pg_CS_name(pg_CS_code(X))
 #endif /* __MULTIBUYTE_H__ */
