@@ -20,6 +20,7 @@
 
 #include "psqlodbc.h"
 
+#if (ODBCVER >= 0x0300)
 #include <stdio.h>
 #include <string.h>
 
@@ -33,10 +34,11 @@ RETCODE		SQL_API
 SQLAllocHandle(SQLSMALLINT HandleType,
 			   SQLHANDLE InputHandle, SQLHANDLE * OutputHandle)
 {
+	CSTR	func = "SQLAllocHandle";
 	RETCODE		ret;
 	ConnectionClass	*conn;
 
-	mylog("[[SQLAllocHandle]]");
+	mylog("[[%s]]", func);
 	switch (HandleType)
 	{
 		case SQL_HANDLE_ENV:
@@ -57,7 +59,7 @@ SQLAllocHandle(SQLSMALLINT HandleType,
 			ENTER_CONN_CS(conn);
 			ret = PGAPI_AllocDesc(InputHandle, OutputHandle);
 			LEAVE_CONN_CS(conn);
-			mylog("Descriptor OutputHandle=%x\n", *OutputHandle);
+inolog("OutputHandle=%x\n", *OutputHandle);
 			break;
 		default:
 			ret = SQL_ERROR;
@@ -70,18 +72,22 @@ SQLAllocHandle(SQLSMALLINT HandleType,
 RETCODE		SQL_API
 SQLBindParam(HSTMT StatementHandle,
 			 SQLUSMALLINT ParameterNumber, SQLSMALLINT ValueType,
-			 SQLSMALLINT ParameterType, SQLUINTEGER LengthPrecision,
+			 SQLSMALLINT ParameterType, SQLULEN LengthPrecision,
 			 SQLSMALLINT ParameterScale, PTR ParameterValue,
-			 SQLINTEGER *StrLen_or_Ind)
+			 SQLLEN *StrLen_or_Ind)
 {
+	CSTR	func = "SQLBindParam";
 	RETCODE			ret;
+	StatementClass	*stmt = (StatementClass *) StatementHandle;
 	int			BufferLength = 512;		/* Is it OK ? */
 
-	mylog("[[SQLBindParam]]");
-	ENTER_STMT_CS((StatementClass *) StatementHandle);
-	SC_clear_error((StatementClass *) StatementHandle);
+	mylog("[[%s]]", func);
+	ENTER_STMT_CS(stmt);
+	SC_clear_error(stmt);
+	StartRollbackState(stmt);
 	ret = PGAPI_BindParameter(StatementHandle, ParameterNumber, SQL_PARAM_INPUT, ValueType, ParameterType, LengthPrecision, ParameterScale, ParameterValue, BufferLength, StrLen_or_Ind);
-	LEAVE_STMT_CS((StatementClass *) StatementHandle);
+	ret = DiscardStatementSvp(stmt,ret, FALSE);
+	LEAVE_STMT_CS(stmt);
 	return ret;
 }
 
@@ -89,32 +95,48 @@ SQLBindParam(HSTMT StatementHandle,
 RETCODE		SQL_API
 SQLCloseCursor(HSTMT StatementHandle)
 {
+	CSTR	func = "SQLCloseCursor";
+	StatementClass	*stmt = (StatementClass *) StatementHandle;
 	RETCODE	ret;
 
-	mylog("[[SQLCloseCursor]]");
-	ENTER_STMT_CS((StatementClass *) StatementHandle);
-	SC_clear_error((StatementClass *) StatementHandle);
+	mylog("[[%s]]", func);
+	ENTER_STMT_CS(stmt);
+	SC_clear_error(stmt);
+	StartRollbackState(stmt);
 	ret = PGAPI_FreeStmt(StatementHandle, SQL_CLOSE);
-	LEAVE_STMT_CS((StatementClass *) StatementHandle);
+	ret = DiscardStatementSvp(stmt,ret, FALSE);
+	LEAVE_STMT_CS(stmt);
 	return ret;
 }
 
 /*	SQLColAttributes -> SQLColAttribute */
-RETCODE		SQL_API
-SQLColAttribute(HSTMT StatementHandle,
-				SQLUSMALLINT ColumnNumber, SQLUSMALLINT FieldIdentifier,
-				PTR CharacterAttribute, SQLSMALLINT BufferLength,
-				SQLSMALLINT *StringLength, PTR NumericAttribute)
+SQLRETURN	SQL_API
+SQLColAttribute(SQLHSTMT StatementHandle,
+			SQLUSMALLINT ColumnNumber,
+			SQLUSMALLINT FieldIdentifier,
+			SQLPOINTER CharacterAttribute,
+			SQLSMALLINT BufferLength,
+			SQLSMALLINT *StringLength,
+#if defined(WITH_UNIXODBC) || defined(WIN32)
+			SQLPOINTER NumericAttribute
+#else
+			SQLLEN *NumericAttribute
+#endif
+			)
 {
+	CSTR	func = "SQLColAttribute";
 	RETCODE	ret;
+	StatementClass	*stmt = (StatementClass *) StatementHandle;
 
-	mylog("[[SQLColAttribute]]");
-	ENTER_STMT_CS((StatementClass *) StatementHandle);
-	SC_clear_error((StatementClass *) StatementHandle);
+	mylog("[[%s]]", func);
+	ENTER_STMT_CS(stmt);
+	SC_clear_error(stmt);
+	StartRollbackState(stmt);
 	ret = PGAPI_ColAttributes(StatementHandle, ColumnNumber,
 					   FieldIdentifier, CharacterAttribute, BufferLength,
 							   StringLength, NumericAttribute);
-	LEAVE_STMT_CS((StatementClass *) StatementHandle);
+	ret = DiscardStatementSvp(stmt,ret, FALSE);
+	LEAVE_STMT_CS(stmt);
 	return ret;
 }
 
@@ -123,10 +145,10 @@ RETCODE		SQL_API
 SQLCopyDesc(SQLHDESC SourceDescHandle,
 			SQLHDESC TargetDescHandle)
 {
+	CSTR	func = "SQLCopyDesc";
 	RETCODE	ret;
 
-	mylog("[[SQLCopyDesc]]\n");
-	DC_clear_error((DescriptorClass *) TargetDescHandle);
+	mylog("[[%s]]\n", func);
 	ret = PGAPI_CopyDesc(SourceDescHandle, TargetDescHandle);
 	return ret;
 }
@@ -136,9 +158,10 @@ RETCODE		SQL_API
 SQLEndTran(SQLSMALLINT HandleType, SQLHANDLE Handle,
 		   SQLSMALLINT CompletionType)
 {
+	CSTR	func = "SQLEndTran";
 	RETCODE	ret;
 
-	mylog("[[SQLEndTran]]");
+	mylog("[[%s]]", func);
 	switch (HandleType)
 	{
 		case SQL_HANDLE_ENV:
@@ -162,7 +185,7 @@ SQLEndTran(SQLSMALLINT HandleType, SQLHANDLE Handle,
 /*	SQLExtendedFetch -> SQLFetchScroll */
 RETCODE		SQL_API
 SQLFetchScroll(HSTMT StatementHandle,
-			   SQLSMALLINT FetchOrientation, SQLINTEGER FetchOffset)
+			   SQLSMALLINT FetchOrientation, SQLLEN FetchOffset)
 {
 	CSTR func = "SQLFetchScroll";
 	StatementClass *stmt = (StatementClass *) StatementHandle;
@@ -174,6 +197,7 @@ SQLFetchScroll(HSTMT StatementHandle,
 	mylog("[[%s]] %d,%d\n", func, FetchOrientation, FetchOffset);
 	ENTER_STMT_CS(stmt);
 	SC_clear_error(stmt);
+	StartRollbackState(stmt);
 	if (FetchOrientation == SQL_FETCH_BOOKMARK)
 	{
 		if (stmt->options.bookmark_ptr)
@@ -184,8 +208,7 @@ mylog("bookmark=%u FetchOffset = %d\n", FetchOffset, bkmarkoff);
 		}
 		else
 		{
-			SC_set_error(stmt, STMT_SEQUENCE_ERROR, "Bookmark isn't specifed yet");
-			SC_log_error(func, "", stmt);
+			SC_set_error(stmt, STMT_SEQUENCE_ERROR, "Bookmark isn't specifed yet", func);
 			ret = SQL_ERROR;
 		}
 	}
@@ -197,6 +220,7 @@ mylog("bookmark=%u FetchOffset = %d\n", FetchOffset, bkmarkoff);
 				pcRow, rowStatusArray, bkmarkoff, opts->size_of_rowset);
 		stmt->transition_status = 6;
 	}
+	ret = DiscardStatementSvp(stmt,ret, FALSE);
 	LEAVE_STMT_CS(stmt);
 	if (ret != SQL_SUCCESS)
 		mylog("%s return = %d\n", func, ret);
@@ -207,8 +231,9 @@ mylog("bookmark=%u FetchOffset = %d\n", FetchOffset, bkmarkoff);
 RETCODE		SQL_API
 SQLFreeHandle(SQLSMALLINT HandleType, SQLHANDLE Handle)
 {
+	CSTR	func = "SQLFreeHandle";
 	RETCODE		ret;
-	mylog("[[SQLFreeHandle]]");
+	mylog("[[%s]]", func);
 	switch (HandleType)
 	{
 		case SQL_HANDLE_ENV:
@@ -240,24 +265,22 @@ SQLGetDescField(SQLHDESC DescriptorHandle,
 	RETCODE	ret;
 
 	mylog("[[SQLGetDescField]]\n");
-	DC_clear_error((DescriptorClass *) DescriptorHandle);
 	ret = PGAPI_GetDescField(DescriptorHandle, RecNumber, FieldIdentifier,
 			Value, BufferLength, StringLength);
 	return ret;
 }
 
-/* TODO - Implement this!! */
+/*	new function */
 RETCODE		SQL_API
 SQLGetDescRec(SQLHDESC DescriptorHandle,
 			  SQLSMALLINT RecNumber, SQLCHAR *Name,
 			  SQLSMALLINT BufferLength, SQLSMALLINT *StringLength,
 			  SQLSMALLINT *Type, SQLSMALLINT *SubType,
-			  SQLINTEGER *Length, SQLSMALLINT *Precision,
+			  SQLLEN *Length, SQLSMALLINT *Precision,
 			  SQLSMALLINT *Scale, SQLSMALLINT *Nullable)
 {
 	mylog("[[SQLGetDescRec]]\n");
 	mylog("Error not implemented\n");
-	DC_clear_error((DescriptorClass *) DescriptorHandle);
 	return SQL_ERROR;
 }
 
@@ -350,13 +373,16 @@ SQLGetStmtAttr(HSTMT StatementHandle,
 {
 	RETCODE	ret;
 	CSTR func = "SQLGetStmtAttr";
+	StatementClass	*stmt = (StatementClass *) StatementHandle;
 
 	mylog("[[%s]] Handle=%u %d\n", func, StatementHandle, Attribute);
-	ENTER_STMT_CS((StatementClass *) StatementHandle);
-	SC_clear_error((StatementClass *) StatementHandle);
+	ENTER_STMT_CS(stmt);
+	SC_clear_error(stmt);
+	StartRollbackState(stmt);
 	ret = PGAPI_GetStmtAttr(StatementHandle, Attribute, Value,
 			BufferLength, StringLength);
-	LEAVE_STMT_CS((StatementClass *) StatementHandle);
+	ret = DiscardStatementSvp(stmt,ret, FALSE);
+	LEAVE_STMT_CS(stmt);
 	return ret;
 }
 
@@ -386,25 +412,25 @@ SQLSetDescField(SQLHDESC DescriptorHandle,
 {
 	RETCODE		ret;
 
-	mylog("[[SQLSetDescField]] h=%u rec=%d field=%d val=%x\n", DescriptorHandle, RecNumber, FieldIdentifier, Value);
-	DC_clear_error((DescriptorClass *) DescriptorHandle);
+	mylog("[[SQLSetDescField]] h=%x rec=%d field=%d val=%x\n", DescriptorHandle, RecNumber, FieldIdentifier, Value);
 	ret = PGAPI_SetDescField(DescriptorHandle, RecNumber, FieldIdentifier,
 				Value, BufferLength);
 	return ret;
 }
 
-/* TODO - Implement this! */
+/*	new fucntion */
 RETCODE		SQL_API
 SQLSetDescRec(SQLHDESC DescriptorHandle,
 			  SQLSMALLINT RecNumber, SQLSMALLINT Type,
-			  SQLSMALLINT SubType, SQLINTEGER Length,
+			  SQLSMALLINT SubType, SQLLEN Length,
 			  SQLSMALLINT Precision, SQLSMALLINT Scale,
-			  PTR Data, SQLINTEGER *StringLength,
-			  SQLINTEGER *Indicator)
+			  PTR Data, SQLLEN *StringLength,
+			  SQLLEN *Indicator)
 {
+	CSTR func = "SQLSetDescRec";
+
 	mylog("[[SQLSetDescRec]]\n");
 	mylog("Error not implemented\n");
-	DC_clear_error((DescriptorClass *) DescriptorHandle);
 	return SQL_ERROR;
 }
 
@@ -422,21 +448,20 @@ SQLSetEnvAttr(HENV EnvironmentHandle,
 	switch (Attribute)
 	{
 		case SQL_ATTR_CONNECTION_POOLING:
-			if (Value == (PTR) SQL_CP_OFF)
+			switch ((SQLUINTEGER) Value)
 			{
-				EN_unset_pooling(env);
-				ret = SQL_SUCCESS;
-			}
+			 	case SQL_CP_OFF:
+					EN_unset_pooling(env);
+					ret = SQL_SUCCESS;
+					break;
 #if defined(WIN_MULTITHREAD_SUPPORT) || defined(POSIX_MULTITHREAD_SUPPORT)
-			else if (Value == (PTR) SQL_CP_ONE_PER_DRIVER)
-			{
-				EN_set_pooling(env);
-				ret = SQL_SUCCESS;
-			}
+				case SQL_CP_ONE_PER_DRIVER:
+					EN_set_pooling(env);
+					ret = SQL_SUCCESS;
+					break;
 #endif /* WIN_MULTITHREAD_SUPPORT */
-			else
-			{
-				ret = SQL_SUCCESS_WITH_INFO;
+				default:
+					ret = SQL_SUCCESS_WITH_INFO;
 			}
 			break;
 		case SQL_ATTR_CP_MATCH:
@@ -444,18 +469,18 @@ SQLSetEnvAttr(HENV EnvironmentHandle,
 			ret = SQL_SUCCESS;
 			break;
 		case SQL_ATTR_ODBC_VERSION:
-			if (Value == (PTR) SQL_OV_ODBC2)
+			if ((SQLUINTEGER) Value == SQL_OV_ODBC2)
 				EN_set_odbc2(env);
 			else
 				EN_set_odbc3(env);
 			ret = SQL_SUCCESS;
 			break;
 		case SQL_ATTR_OUTPUT_NTS:
-			if (Value == (PTR) SQL_TRUE)
+			if ((SQLUINTEGER) Value == SQL_TRUE)
 				ret = SQL_SUCCESS;
 			else
 				ret = SQL_SUCCESS_WITH_INFO;
-
+	
 			break;
 		default:
 			env->errornumber = CONN_INVALID_ARGUMENT_NO;
@@ -480,10 +505,12 @@ SQLSetStmtAttr(HSTMT StatementHandle,
 	StatementClass *stmt = (StatementClass *) StatementHandle;
 	RETCODE	ret;
 
-	mylog("[[%s]] Handle=%u %d,%u\n", func, StatementHandle, Attribute, Value);
+	mylog("[[%s]] Handle=%x %d,%u\n", func, StatementHandle, Attribute, Value);
 	ENTER_STMT_CS(stmt);
 	SC_clear_error(stmt);
+	StartRollbackState(stmt);
 	ret = PGAPI_SetStmtAttr(StatementHandle, Attribute, Value, StringLength);
+	ret = DiscardStatementSvp(stmt,ret, FALSE);
 	LEAVE_STMT_CS(stmt);
 	return ret;
 }
@@ -493,11 +520,12 @@ SQLSetStmtAttr(HSTMT StatementHandle,
 			|= (1 << ((uwAPI) & 0x000F)) \
 				)
 RETCODE		SQL_API
-PGAPI_GetFunctions30(HDBC hdbc, UWORD fFunction, UWORD FAR * pfExists)
+PGAPI_GetFunctions30(HDBC hdbc, SQLUSMALLINT fFunction, SQLUSMALLINT FAR * pfExists)
 {
 	ConnectionClass	*conn = (ConnectionClass *) hdbc;
 	ConnInfo	*ci = &(conn->connInfo);
 
+inolog("lie=%d\n", ci->drivers.lie);
 	CC_clear_error(conn);
 	if (fFunction != SQL_API_ODBC3_ALL_FUNCTIONS)
 		return SQL_ERROR;
@@ -512,7 +540,7 @@ PGAPI_GetFunctions30(HDBC hdbc, UWORD fFunction, UWORD FAR * pfExists)
 	 * i);
 	 */
 	SQL_FUNC_ESET(pfExists, SQL_API_SQLBINDCOL);		/* 4 */
-	SQL_FUNC_ESET(pfExists, SQL_API_SQLCANCEL); /* 5 */
+	SQL_FUNC_ESET(pfExists, SQL_API_SQLCANCEL);		/* 5 */
 	SQL_FUNC_ESET(pfExists, SQL_API_SQLCOLATTRIBUTE);	/* 6 */
 	SQL_FUNC_ESET(pfExists, SQL_API_SQLCONNECT);		/* 7 */
 	SQL_FUNC_ESET(pfExists, SQL_API_SQLDESCRIBECOL);	/* 8 */
@@ -551,14 +579,14 @@ PGAPI_GetFunctions30(HDBC hdbc, UWORD fFunction, UWORD FAR * pfExists)
 	/* SQL_FUNC_ESET(pfExists, SQL_API_SQLSETSTMTOPTION); 51 deprecated */
 	SQL_FUNC_ESET(pfExists, SQL_API_SQLSPECIALCOLUMNS);	/* 52 */
 	SQL_FUNC_ESET(pfExists, SQL_API_SQLSTATISTICS);		/* 53 */
-	SQL_FUNC_ESET(pfExists, SQL_API_SQLTABLES); /* 54 */
+	SQL_FUNC_ESET(pfExists, SQL_API_SQLTABLES);		/* 54 */
 	if (ci->drivers.lie)
-		SQL_FUNC_ESET(pfExists, SQL_API_SQLBROWSECONNECT); /* 55 not implmented yet */
+		SQL_FUNC_ESET(pfExists, SQL_API_SQLBROWSECONNECT);	/* 55 */
 	if (ci->drivers.lie)
-		SQL_FUNC_ESET(pfExists, SQL_API_SQLCOLUMNPRIVILEGES); /* 56 not implemented yet */
+		SQL_FUNC_ESET(pfExists, SQL_API_SQLCOLUMNPRIVILEGES);	/* 56 */
 	SQL_FUNC_ESET(pfExists, SQL_API_SQLDATASOURCES);	/* 57 */
-	if (ci->drivers.lie)
-		SQL_FUNC_ESET(pfExists, SQL_API_SQLDESCRIBEPARAM); /* 58 not properly implemented */
+	if (SUPPORT_DESCRIBE_PARAM(ci) || ci->drivers.lie)
+		SQL_FUNC_ESET(pfExists, SQL_API_SQLDESCRIBEPARAM); /* 58 */
 	SQL_FUNC_ESET(pfExists, SQL_API_SQLEXTENDEDFETCH); /* 59 deprecated ? */
 
 	/*
@@ -571,11 +599,11 @@ PGAPI_GetFunctions30(HDBC hdbc, UWORD fFunction, UWORD FAR * pfExists)
 	SQL_FUNC_ESET(pfExists, SQL_API_SQLNUMPARAMS);		/* 63 */
 	/* SQL_FUNC_ESET(pfExists, SQL_API_SQLPARAMOPTIONS); 64 deprecated */
 	SQL_FUNC_ESET(pfExists, SQL_API_SQLPRIMARYKEYS);	/* 65 */
-	SQL_FUNC_ESET(pfExists, SQL_API_SQLPROCEDURECOLUMNS);	/* 66 */
+	SQL_FUNC_ESET(pfExists, SQL_API_SQLPROCEDURECOLUMNS);	/* 66 */ 
 	SQL_FUNC_ESET(pfExists, SQL_API_SQLPROCEDURES);		/* 67 */
 	SQL_FUNC_ESET(pfExists, SQL_API_SQLSETPOS);		/* 68 */
 	/* SQL_FUNC_ESET(pfExists, SQL_API_SQLSETSCROLLOPTIONS); 69 deprecated */
-	SQL_FUNC_ESET(pfExists, SQL_API_SQLTABLEPRIVILEGES);		/* 70 */
+	SQL_FUNC_ESET(pfExists, SQL_API_SQLTABLEPRIVILEGES);	/* 70 */
 	/* SQL_FUNC_ESET(pfExists, SQL_API_SQLDRIVERS); */	/* 71 */
 	SQL_FUNC_ESET(pfExists, SQL_API_SQLBINDPARAMETER);	/* 72 */
 
@@ -604,7 +632,7 @@ PGAPI_GetFunctions30(HDBC hdbc, UWORD fFunction, UWORD FAR * pfExists)
 	SQL_FUNC_ESET(pfExists, SQL_API_SQLSETENVATTR);		/* 1019 */
 	SQL_FUNC_ESET(pfExists, SQL_API_SQLSETSTMTATTR);	/* 1020 */
 	SQL_FUNC_ESET(pfExists, SQL_API_SQLFETCHSCROLL);	/* 1021 */
-	if (ci->updatable_cursors)
+	if (0 != ci->updatable_cursors)
 		SQL_FUNC_ESET(pfExists, SQL_API_SQLBULKOPERATIONS);	/* 24 */
 
 	return SQL_SUCCESS;
@@ -618,17 +646,12 @@ SQLBulkOperations(HSTMT hstmt, SQLSMALLINT operation)
 	StatementClass	*stmt = (StatementClass *) hstmt;
 
 	ENTER_STMT_CS(stmt);
-#ifndef	DRIVER_CURSOR_IMPLEMENT
-	SC_set_error(stmt, STMT_NOT_IMPLEMENTED_ERROR,
-	"driver must be compiled with the DRIVER_CURSOR_IMPLEMENT option");
-	SC_log_error(func, "", stmt);
-	ret = SQL_ERROR;
-#else
-	mylog("[[%s]] Handle=%u %d\n", func, hstmt, operation);
-	SC_clear_error((StatementClass *) hstmt);
+	mylog("[[%s]] Handle=%x %d\n", func, hstmt, operation);
+	SC_clear_error(stmt);
+	StartRollbackState(stmt);
 	ret = PGAPI_BulkOperations(hstmt, operation);
-#endif /* DRIVER_CURSOR_IMPLEMENT */
+	ret = DiscardStatementSvp(stmt,ret, FALSE);
 	LEAVE_STMT_CS(stmt);
 	return ret;
-}
-
+}	
+#endif /* ODBCVER >= 0x0300 */
