@@ -13,10 +13,12 @@
 #include <errno.h>
 
 #ifndef WIN32
+#define	WSAAPI
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <sys/un.h>
 #include <unistd.h>
+#include <fcntl.h>
 #include <netdb.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
@@ -34,26 +36,82 @@ typedef unsigned int in_addr_t;
 
 #define SOCK_ERRNO	errno
 #define SOCK_ERRNO_SET(e)	(errno = e)
-#if defined HAVE_SYS_UN_H && !defined HAVE_UNIX_SOCKETS
+#ifdef	HAVE_SYS_UN_H
 #define HAVE_UNIX_SOCKETS
 #endif /* HAVE_SYS_UN_H */
 #else
-#include <winsock.h>
-#define SOCKETFD SOCKET
-#define SOCK_ERRNO	(WSAGetLastError())
-#define SOCK_ERRNO_SET(e)	WSASetLastError(e)
-#endif /* WIN32 */
 
-#define SOCKET_ALREADY_CONNECTED			1
-#define SOCKET_HOST_NOT_FOUND				2
+#include <winsock2.h>
+#include <ws2tcpip.h>
+
+#if defined(_MSC_VER) && (_MSC_VER < 1300)
+/*
+ *      The order of the structure elements on Win32 doesn't match the
+ *      order specified in the standard, but we have to match it for
+ *      IPv6 to work.
+ */
+
+#define AI_PASSIVE     0x1  // Socket address will be used in bind() call.
+#define AI_CANONNAME   0x2  // Return canonical name in first ai_canonname.
+#define AI_NUMERICHOST 0x4  // Nodename must be a numeric address string.
+
+#define _SS_MAXSIZE 128
+#define _SS_ALIGNSIZE (sizeof(__int64))
+#define _SS_PAD1SIZE (_SS_ALIGNSIZE - sizeof (short))
+#define _SS_PAD2SIZE (_SS_MAXSIZE - (sizeof (short) + _SS_PAD1SIZE + _SS_ALIGNSIZE))
+
+typedef int socklen_t;
+
+struct sockaddr_storage {
+	short ss_family;
+	char __ss_pad1[_SS_PAD1SIZE];
+	__int64 __ss_align;
+	char __ss_pad2[_SS_PAD2SIZE];
+};
+struct addrinfo
+{
+	int		ai_flags;
+	int		ai_family;
+	int		ai_socktype;
+	int		ai_protocol;
+	size_t	ai_addrlen;
+	char	*ai_canonname;
+	struct sockaddr *ai_addr;
+	struct addrinfo *ai_next;
+};
+#endif /* _MSC_VER */
+
+#define SOCKETFD SOCKET
+#define SOCK_ERRNO		(WSAGetLastError())
+#define SOCK_ERRNO_SET(e)	WSASetLastError(e)
+#ifndef	EWOULDBLOCK
+#define	EWOULDBLOCK	WSAEWOULDBLOCK
+#endif /* EWOULDBLOCK */
+#ifndef	ECONNRESET
+#define	ECONNRESET	WSAECONNRESET
+#endif /* ECONNRESET */
+#ifndef	EINPROGRESS
+#define	EINPROGRESS	WSAEINPROGRESS
+#endif /* EINPROGRESS */
+#endif /* WIN32 */
+typedef void (WSAAPI *freeaddrinfo_func) (struct addrinfo *); 
+typedef int (WSAAPI *getaddrinfo_func) (const char *, const char *,
+	const struct addrinfo *, struct addrinfo **); 
+typedef int (WSAAPI *getnameinfo_func) (const struct sockaddr *,
+	socklen_t, char *, DWORD, char *, DWORD, int);
+
+#define SOCKET_ALREADY_CONNECTED		1
+#define SOCKET_HOST_NOT_FOUND			2
 #define SOCKET_COULD_NOT_CREATE_SOCKET		3
-#define SOCKET_COULD_NOT_CONNECT			4
-#define SOCKET_READ_ERROR					5
-#define SOCKET_WRITE_ERROR					6
+#define SOCKET_COULD_NOT_CONNECT		4
+#define SOCKET_READ_ERROR			5
+#define SOCKET_WRITE_ERROR			6
 #define SOCKET_NULLPOINTER_PARAMETER		7
-#define SOCKET_PUT_INT_WRONG_LENGTH			8
-#define SOCKET_GET_INT_WRONG_LENGTH			9
-#define SOCKET_CLOSED						10
+#define SOCKET_PUT_INT_WRONG_LENGTH		8
+#define SOCKET_GET_INT_WRONG_LENGTH		9
+#define SOCKET_CLOSED				10
+#define SOCKET_READ_TIMEOUT			11
+#define SOCKET_WRITE_TIMEOUT			12
 
 
 struct SocketClass_
@@ -67,12 +125,17 @@ struct SocketClass_
 	UCHAR *buffer_out;
 
 	SOCKETFD	socket;
+	unsigned int	pversion;
+	int		reslen;
 
-	char	   *errormsg;
-	int			errornumber;
-	struct sockaddr	*sadr; /* Used for handling connections for cancel */
+	char		*errormsg;
+	int		errornumber;
 	int		sadr_len;
-	struct sockaddr_in sadr_in; /* Used for INET connections */
+	struct sockaddr_storage sadr_area; /* Used for various connections */
+	/* SSL stuff */
+	void		*ssl;		/* libpq ssl */
+	void		*pqconn;	/* libpq PGConn */
+	BOOL		via_libpq;	/* using libpq library ? */
 
 	char		reverse;		/* used to handle Postgres 6.2 protocol
 								 * (reverse byte order) */
@@ -121,21 +184,19 @@ struct SocketClass_
 /* Socket prototypes */
 SocketClass *SOCK_Constructor(const ConnectionClass *conn);
 void		SOCK_Destructor(SocketClass *self);
-char		SOCK_connect_to(SocketClass *self, unsigned short port,
-				char *hostname
-#ifdef HAVE_UNIX_SOCKETS
-				, char *uds
-#endif
-				);
+char		SOCK_connect_to(SocketClass *self, unsigned short port, char *hostname, long timeout);
+int		SOCK_get_id(SocketClass *self);
 void		SOCK_get_n_char(SocketClass *self, char *buffer, int len);
 void		SOCK_put_n_char(SocketClass *self, char *buffer, int len);
 BOOL		SOCK_get_string(SocketClass *self, char *buffer, int bufsize);
-void		SOCK_put_string(SocketClass *self, char *string);
+void		SOCK_put_string(SocketClass *self, const char *string);
 int			SOCK_get_int(SocketClass *self, short len);
 void		SOCK_put_int(SocketClass *self, int value, short len);
 void		SOCK_flush_output(SocketClass *self);
 UCHAR		SOCK_get_next_byte(SocketClass *self);
 void		SOCK_put_next_byte(SocketClass *self, UCHAR next_byte);
+int		SOCK_get_response_length(SocketClass *self);
 void		SOCK_clear_error(SocketClass *self);
+UInt4		SOCK_skip_n_bytes(SocketClass *self, UInt4 skip_length);
 
 #endif /* __SOCKET_H__ */
