@@ -10,7 +10,9 @@
 #include "psqlodbc.h"
 #include <stdio.h>
 #include <string.h>
-#include <stdlib.h>
+#ifdef	WIN32
+#include <malloc.h>
+#endif /* WIN32 */
 
 #define	byte3check	0xfffff800
 #define	byte2_base	0x80c0
@@ -41,14 +43,17 @@ UInt4	ucs2strlen(const SQLWCHAR *ucs2str)
 		;
 	return len;
 }
-char *ucs2_to_utf8(const SQLWCHAR *ucs2str, Int4 ilen, UInt4 *olen, BOOL lower_identifier)
+char *ucs2_to_utf8(const SQLWCHAR *ucs2str, Int4 ilen, Int4 *olen, BOOL lower_identifier)
 {
 	char *	utf8str;
 /*mylog("ucs2_to_utf8 %x ilen=%d ", ucs2str, ilen);*/
 
 	if (!ucs2str)
+	{
+		*olen = SQL_NULL_DATA;
 		return NULL;
-	if (ilen < 0)
+	}
+	if (SQL_NTS == ilen)
 		ilen = ucs2strlen(ucs2str);
 /*mylog(" newlen=%d", ilen);*/
 	utf8str = (char *) malloc(ilen * 3 + 1);
@@ -76,7 +81,7 @@ char *ucs2_to_utf8(const SQLWCHAR *ucs2str, Int4 ilen, UInt4 *olen, BOOL lower_i
 					    ((byte2_mask1 & *wstr) >> 6) |
 					    ((byte2_mask2 & *wstr) << 8);
 				memcpy(utf8str + len, (char *) &byte2code, sizeof(byte2code));
-				len += 2; 
+				len += sizeof(byte2code); 
 			}
 			else
 			{
@@ -121,11 +126,11 @@ UInt4	utf8_to_ucs2_lf(const char *utf8str, Int4 ilen, BOOL lfconv, SQLWCHAR *ucs
 	{
 		if (iswascii(*str))
 		{
-			if (lfconv && *str == '\n' &&
-			    (i == 0 || str[-1] != '\r'))
+			if (lfconv && PG_LINEFEED == *str &&
+			    (i == 0 || PG_CARRIAGE_RETURN != str[-1]))
 			{
 				if (ocount < bufcount)
-					ucs2str[ocount] = '\r';
+					ucs2str[ocount] = PG_CARRIAGE_RETURN;
 				ocount++;
 			}
 			if (ocount < bufcount)
